/** defines and runs a simple socket based text client
 */
import java.net.*;
import java.io.*;
import java.util.ArrayList;
import simsa.xml.XMLParser;

public class StressTestClient {
  Socket client;
  BufferedWriter sockOut;
  //BufferedReader sockIn;
  InputStream sockIn;
  BufferedReader sysIn;
  Thread get, put;
  boolean valid;
  int myID;
  XMLParser xp;
  ArrayList<String> vo = new ArrayList<String>();
  
  /** constructor creates connection to server with TCP
   *  on the specified address and port.
   *  defines some buffered reader/writer objects and
   *  starts the actual client
   */
  public StressTestClient(String address, int port, int id, String mod) throws Exception {
    myID = id;
    valid = true; //used to kill the threads and close the client
    
    vo.add("_msg");     //0
    vo.add("command"); //1
    vo.add("who"); //1
    vo.add("where"); //1
    vo.add("conack"); //1
    vo.add("error"); //1
    xp = new XMLParser(vo);
    
    makeConnection(address, port);
    sockOut = new BufferedWriter(new OutputStreamWriter(client.getOutputStream()));
    //sockIn = new BufferedReader(new InputStreamReader(client.getInputStream()));
    
    sockIn = client.getInputStream();
    
    sysIn = new BufferedReader(new InputStreamReader(System.in));
    writeToStream("<connect><name>ST" + id + "</name><mid>" + mod + "</mid></connect>");
    runClient();
  }
  
  /** attempts to connect to the server on the specified address and port
   */
  private void makeConnection(String address, int port) throws Exception {
    client = new Socket(InetAddress.getByName(address), port);
  }
  
  /** defines the main I/O threads and starts them
   */
  private void runClient() throws Exception {
    //  gets strings from the server
    get = new Thread(new Runnable() {
      public void run() {
        try {
          while(valid) {
            xp.parse((char)sockIn.read());
            if(xp.hasData()) {
              String data = xp.getData();
              switch(xp.getTagType()) {
                case 0:  //_msg
                  if(data.indexOf('%') >= 0)
                    writeToStream("<_msg>I am " + myID + "</_msg>");
                  System.out.print(data);
                  break;
                case 1:  //command
                  if(data.equalsIgnoreCase("close")) {
                    valid = false;
                    System.out.println("Received Commmand \'Close\' from Server");
                  }
                  break;
                default:
                  System.out.print(xp.getTag() + "=" + data);
              }
              xp.reset();
            }
          }
        } catch(Exception e) {
          System.out.println("Connection to server terminated.");
        } finally {
          try {
            client.close();
          } catch(Exception e) {
            System.out.println("Client cannot exit cleanly.");
          }
          System.exit(1);
        }
      }
    });
    
    //  writes to the server
    put = new Thread(new Runnable() {
      public void run() {
        try {
          while(valid)
            writeToStream("<_msg>" + sysIn.readLine() + "</_msg>");
        } catch(Exception e) {
          System.out.println("Connection to server terminated.");
        } finally {
          try {
            client.close();
          } catch(Exception e) {
            System.out.println("Client cannot exit cleanly.");
          }
          System.exit(1);
        }
      }
    });
    
    get.start();
    put.start();
  }
  
  /** writes to the socket output stream to the server
   */
  private void writeToStream(String in) throws Exception {
    sockOut.write(in);
    sockOut.flush();
  }
  
  /** creates an instance of a client and catches exceptions
   */
  public static void main(String[] args) {
    try {
      StressTestClient tc = new StressTestClient(args[0].trim(), Integer.valueOf(args[1].trim()), Integer.valueOf(args[2].trim()),  args[3].trim());
    } catch(Exception e) {
      System.out.println("The client has encountered a fatal exception: " + e);
    }
  }
}
