package Spong;
import java.net.*;
import java.io.*;
import java.util.ArrayList;
import simsa.xml.XMLParser;

public class SpongCli {
  Socket client;
  BufferedWriter sockOut;
  InputStream sockIn;
  BufferedReader sysIn;
  Thread get, put;
  boolean valid;
  XMLParser xp;
  XMLParser ixp;
  
  MainScreen ms = new MainScreen();
  
  ArrayList<String> vo = new ArrayList<String>();
  ArrayList<String> vi = new ArrayList<String>();
  
  
  /** constructor creates connection to server with TCP
   *  on the specified address and port.
   *  defines some buffered reader/writer objects and
   *  starts the actual client
   */
  public SpongCli(String address, int port) throws Exception {
    valid = true; //used to kill the threads and close the client
    vo.add("_msg");     //0
    vo.add("command"); //1
    vo.add("who"); //1
    vo.add("where"); //1
    vo.add("conack"); //1
    vo.add("error"); //1
    xp = new XMLParser(vo);
    
    makeConnection(address, port);
    sockOut = new BufferedWriter(new OutputStreamWriter(client.getOutputStream()));
    
    sockIn = client.getInputStream();
    
    sysIn = new BufferedReader(new InputStreamReader(System.in));
    
    runClient();
    ms.setVisible(true);
  }
  
  /** attempts to connect to the server on the specified address and port
   */
  private void makeConnection(String address, int port) throws Exception {
    client = new Socket(InetAddress.getByName(address), port);
  }
  
  /** defines the main I/O threads and starts them
   */
  private void runClient() throws Exception {
    //  gets strings from the server
    get = new Thread(new Runnable() {
      public void run() {
        vi.add("d");
        vi.add("s");
        
        ixp = new XMLParser(vi);
        
        String[] coords = new String[4];
        
        int x = 0;
        int y = 0;
        int p1 = 0;
        int p2 = 0;
        
        try {
          while(valid) {
            xp.parse((char)sockIn.read());
            if(xp.hasData()) {
              String data = xp.getData();
              switch(xp.getTagType()) {
                case 0:  //_msg
                  for(char c : data.toCharArray()) {
                    ixp.parse(c);
                    if(ixp.hasData()) {
                      String innerData = ixp.getData();
                      switch (ixp.getTagType()) {
                        case 0:
                          coords = innerData.split(",");
                          x = Integer.parseInt(coords[0]);
                          y = Integer.parseInt(coords[1]);
                          p1 = Integer.parseInt(coords[2]);
                          p2 = Integer.parseInt(coords[3]);
                          ms.setScreenElms(x, y, p1, p2);
                          break;
                        case 1:
                          coords = innerData.split(",");
                          x = Integer.parseInt(coords[0]);
                          y = Integer.parseInt(coords[1]);
                          ms.setScore(x, y);
                          break;
                      }
                      ixp.reset();
                    }
                  }
                  break;
                case 1:  //command
                  if(data.equalsIgnoreCase("close")) {
                    valid = false;
                    System.out.println("Received Commmand \'Close\' from Server");
                  }
                  break;
                default:
                  System.out.print(xp.getTag() + "=" + data);
              }
              xp.reset();
            }
          }
        } catch(Exception e) {
          System.out.println("Connection to server terminated.");
        } finally {
          try {
            client.close();
          } catch(Exception e) {
            System.out.println("Client cannot exit cleanly.");
          }
          System.exit(1);
        }
      }
    });
    
    //  writes to the server
    put = new Thread(new Runnable() {
      public void run() {
        try {
          while(valid)
            writeToStream(sysIn.readLine());
        } catch(Exception e) {
          System.out.println("Connection to server terminated.");
        } finally {
          try {
            client.close();
          } catch(Exception e) {
            System.out.println("Client cannot exit cleanly.");
          }
          System.exit(1);
        }
      }
    });
    
    get.start();
    put.start();
  }
  
  /** writes to the socket output stream to the server
   */
  private void writeToStream(String in) throws Exception {
    sockOut.write(in);
    sockOut.flush();
  }
  
  /** creates an instance of a client and catches exceptions
   *  SENDING Command line args with IDE project properties
   */
  public static void main(String[] args) {
    try {
      if(args.length == 2) {
        SpongCli tc = new SpongCli(args[0].trim(), Integer.valueOf((args[1].trim())));
      } else {
        //read IP address from stdin
        byte[] tempIn = new byte[32];
        System.out.print("Enter the IP address of the server: ");
        System.in.read(tempIn);
        String addr = new String(tempIn).trim();
        
        //read port number from stdin
        tempIn = new byte[16];
        System.out.print("Enter the port to connect on: ");
        System.in.read(tempIn);
        int sock = Integer.valueOf(new String(tempIn).trim());
        
        SpongCli tc = new SpongCli(addr, sock);
      }
    } catch(Exception e) {
      System.out.println("The client has encountered a fatal exception: " + e);
    }
  }
}
