package simsa.xml;
//Rebuilt 7/22/06 with StringBuilder for efficiency
import java.util.ArrayList;

public class XMLParser {
  
  private StringBuilder buffer;
  private StringBuilder tag;
  private int tagType;
  private boolean inTag;
  private boolean inBuffer;
  private boolean parseComplete;
  private ArrayList<String> validTagList;
  
  public XMLParser(ArrayList<String> inputList) {
    reset();
    validTagList = inputList;
  }
  
  private StringBuilder removeFromEnd(StringBuilder s, int num) {
    return s.delete(s.length() - num, s.length());
  }
  private int isValidTag(StringBuilder s) {
    return validTagList.indexOf(s.toString().toLowerCase());
  }
  
  
  public void parse(char temp) {
    if(inBuffer) { //save data between valid tag
      buffer.append(temp);
    }
    if(inTag) { //save data between opening and closing brackets
      tag.append(temp);
    }
    if(inTag && temp == '<') { //reset tagSearch if we find new opening bracket
      reset();
      inTag = true;
    }
    if(temp == '<' && !inTag && !inBuffer) { //start saving tag if we find bracket
      inTag = true;
    }
    if(temp == '>' && inTag) { //validate tag on closing bracket
      tag = removeFromEnd(tag, 1);
      tagType = isValidTag(tag); //returns -1 on invalid tag
      if(tagType >= 0 ) {
        inTag = false;
        inBuffer = true;
      } else {
        reset();
      }
    }
    if(inBuffer && hasEndTag(buffer)) { //stop saving data on end tag
      inBuffer = false;
      buffer = removeFromEnd(buffer, tag.length() + 3);
      parseComplete = true;
    }
  }
  
  private boolean hasEndTag(StringBuilder s) {
    return s.toString().toLowerCase().endsWith("</" + tag.toString().toLowerCase() + ">");
  }
  
  public void reset() {
    buffer = new StringBuilder();
    tag = new StringBuilder();
    tagType = -1;
    inTag = false;
    inBuffer = false;
    parseComplete = false;
  }
  public boolean hasData() {
    return parseComplete;
  }
  public String getData() {
    return buffer.toString();
  }
  public Integer getDataAsInt() {
    Integer retVal;
    try {
      retVal = Integer.parseInt(buffer.toString());
    } catch (Exception e) {
      retVal = 0;
    }
    return retVal;
  }
  public int getTagType() {
    return tagType;
  }
  public String getTag() {
    return tag.toString();
  }
}