package Modules;

import java.io.InputStream;
import java.io.OutputStream;
import java.net.Socket;
import java.util.ArrayList;
import java.util.HashMap;
import simsa.module.Module;
import simsa.xml.XMLParser;

public class SmartChat extends Module {
  
  InputStream is;
  OutputStream os;
  XMLParser oxp, ixp;
  
  ArrayList<String> vo = new ArrayList<String>();
  ArrayList<String> vi = new ArrayList<String>();
  HashMap<Integer, String> names = new HashMap<Integer, String>();
  
  public void init() {
    //valid messages when not connected
    vo.add("_pkt");      //0
    vo.add("command");   //1
    vi.add("from");      //0
    vi.add("_msg");       //1
    
    oxp = new XMLParser(vo);
    ixp = new XMLParser(vi);
  }
  
  public void preTerminate() {
    System.out.println("Smart Chat is exiting...");
  }
  
  public void run() {
    String temp;
    
    try {
      connection.getOutputStream().write("<connect><mtype>Chat</mtype><name>Smart Chat</name></connect>".getBytes());
    } catch (Exception e) {
      running = false;
    }
    
    try {
      is = connection.getInputStream();
      os = connection.getOutputStream();
    } catch (Exception e) {
      System.out.println("Error opening streams...");
      running = false;
    }
    while(running) {
      try {
        oxp.parse((char)is.read());
        if(oxp.hasData()) {
          temp = oxp.getData();
          switch(oxp.getTagType()) {
            case 0: //_msg
            {
              int tempFrom = 0;
              String tempMsg = "";
              for(char c : temp.toCharArray()) { //process inner tags
                ixp.parse(c);
                if(ixp.hasData()) {
                  switch(ixp.getTagType()) {
                    case 0:
                      tempFrom = ixp.getDataAsInt();
                      break;
                    case 1:
                      tempMsg = ixp.getData();
                      break;
                  }
                  ixp.reset();
                }
              }
             String outbound = "<_pkt><to>!" + tempFrom + "</to><_msg><id>" + tempFrom;
              outbound += "</id><said>" + getName(tempFrom) + ": " + tempMsg + "</said></_msg></_pkt>";
              os.write(outbound.getBytes());
            }
            break;
            case 1: //command
            {
              if(temp.equalsIgnoreCase("close"))
                terminate();
            }
            break;
          }
          oxp.reset();
        }
      } catch (Exception e) {
        terminate();
      }
    }
  }
  
  public String getName(int id) {
    if(id <= 0)
      return "";
      
    String temp = names.get(id);
    if(temp == null) {
      try {
        //do name lookup at server
        int readByte;
        
        int tempId = 0;
        String tempName = "";
        
        //set up name parser
        ArrayList<String> tags = new ArrayList<String>();
        tags.add("id");   //0
        tags.add("name"); //1
        XMLParser nameParser = new XMLParser(tags);
        
        Socket s = new Socket(connectedHost, 12345);
        String whoisStr = "<whois><id>" + id + "</id></whois>";
        s.getOutputStream().write(whoisStr.getBytes());
        while((readByte = s.getInputStream().read()) != -1) {
          nameParser.parse((char)readByte);
          if(nameParser.hasData()) {
            switch(nameParser.getTagType()) {
              case 0:
                tempId = nameParser.getDataAsInt();
                break;
              case 1:
                tempName = nameParser.getData();
                break;
            }
            nameParser.reset();
          }
          if(tempId != 0 && !tempName.equals("")) {
            break;
          }
        }
        names.put(tempId, tempName);
        temp = tempName;
      } catch (Exception e) {
        System.out.println("Error during name lookup.");
      }
    }
    return temp;
  }

  public static void main(String[] args) {
    try {
      SmartChat c = new SmartChat();
      c.connect(args[0].trim(), Integer.parseInt(args[1].trim()));
      Thread modRun = new Thread(c); //actually starts the module
      modRun.start();
    } catch (Exception e) {
      System.err.println("Module failed to load.");
    }
  }
}
