package Server;
import java.nio.ByteBuffer;
import java.nio.channels.SelectionKey;
import simsa.xml.XMLParser;
import java.util.ArrayList;
/** Contains all of the state information for a SIMSA client
 *  as well as handles inbound messages according to client
 *  specifications with a client parser. 2006/08/10 Kyle Reed
 */
public class ClientContainer extends ConnectionContainer {
  private ModuleContainer moduleCtnr;
  private XMLParser oxp, ixp;
  
  private ArrayList<String> vo_c = new ArrayList<String>();
  private ArrayList<String> vo_n = new ArrayList<String>();
  private ArrayList<String> vi_c = new ArrayList<String>();
  private ArrayList<String> vi_n = new ArrayList<String>();
  
  //CONSTRUCTORS
  /** Creates a client container for an incoming connection.
   *  See Connection Container documentation.
   * 2006/08/10 Kyle Reed
   */
  public ClientContainer(SelectionKey key) {
    super(key);
    moduleCtnr = null;
    connected = false;
    
    //valid messages when connected
    vo_c.add("_msg"); //0
    
    //valid messages when not connected
    vo_n.add("connect");    //0
    vo_n.add("whois");      //1
    vo_n.add("whereis");    //2
    vo_n.add("disconnect"); //3
    vi_n.add("name");  //0
    vi_n.add("id");    //1
    vi_n.add("mtype"); //2
    vi_n.add("mid");   //3
    vi_n.add("key");   //4
    
    oxp = new XMLParser(vo_n);
    ixp = new XMLParser(vi_n);
  }
  
  //MODULE FUNCTIONS
  /** Associates a module with this client for routing messages.
   *  2006/08/10 Kyle Reed
   */
  public void connectToModule(ModuleContainer mc) {
    if(moduleCtnr == null) {
      moduleCtnr = mc;
      mc.registerClient(this);
    }
  }
  /** Removes this client from the registry of its
   *  associated modules. 2006/08/10 Kyle Reed
   */
  public void disconnectFromModule() {
    if(moduleCtnr != null)
      moduleCtnr.removeClient(idNum);
    moduleCtnr = null;
  }
  
  /** Implements the message handling section of a client.
   *  This functions processes incoming data accoring to the
   *  SIMSA spec for clients. 2006/08/10 Kyle Reed
   */
  public void handleInput() {
    bb = ByteBuffer.allocate(server.MAX_BUFFER_SIZE);
    bb.clear();
    int readNum;
    try {
      readNum = io.read(bb);
      bb.flip();
      ba = bb.array();
      
      //The read function returns -1 on EOF (Stream closed)
      if(readNum == -1) throw new Exception("Connection Closed.");
      
      for(int i = 0; i < readNum; i++) {
        oxp.parse((char)ba[i]);
        if(oxp.hasData()) {
          String data = oxp.getData();
          if(connected) {
            switch (oxp.getTagType()) {
              case 0: //_msg
                moduleCtnr.writeToSelf(
                    "<_pkt><from>" + idNum +
                    "</from><_msg>" + data +
                    "</_msg></_pkt>");
                break;
            }
          } else {
            switch (oxp.getTagType()) {
              case 0: //connect
              {
                String tempName = "";
                int chNum = 0;
                int tempMID = 0;
                for(char inner_temp : data.toCharArray()) {
                  ixp.parse(inner_temp);
                  if(ixp.hasData()) {
                    switch(ixp.getTagType()) {
                      case 0: //name
                        tempName = ixp.getData();
                        break;
                      case 3: //MID
                        tempMID = ixp.getDataAsInt();
                        break;
                      case 4: //chNum
                        chNum = ixp. getDataAsInt();
                        break;
                    }
                    ixp.reset();
                  }
                }
                ixp.reset();
                
                if(!tempName.equals("") && tempMID != 0) {
                  //validate unique names
                  //or name and key
                  //fix error msg for chNum
                  
                  //initialize new parsers
                  oxp = new XMLParser(vo_c);
                  ixp = new XMLParser(vi_c);
                  connected = true;
                  idName = tempName;
                  if(server.moduleList.containsKey(tempMID)) {
                    ModuleContainer mc = server.moduleList.get(tempMID);
                    connectToModule(mc);
                    writeToSelf("<conack><id>" + idNum + "</id>" +
                        "<key>" + challengeNum + "</key></conack>");
                  } else {
                    //fix the error message
                    writeToSelf("<error><mid>" + tempMID + "</mid></error>");
                  }
                } else {
                  //fix the error message for chNum
                  if(tempName.equals(""))
                    writeToSelf("<error><name></name></error>");
                  if(tempMID == 0)
                    writeToSelf("<error><mid></mid></error>");
                }
              }
              break;
              case 1: //whois
              {
                String lookups = "";
                for(char inner_temp : data.toCharArray()) {
                  ixp.parse(inner_temp);
                  if(ixp.hasData()) {
                    switch(ixp.getTagType()) {
                      case 1: //id
                        lookups = ixp.getData();
                        break;
                    }
                    ixp.reset();
                  }
                }
                ixp.reset();
                
                if(!lookups.equals("")) {
                  if(lookups.charAt(0) == '0') {
                    for(ClientContainer tempCC : server.clientList.values()) {
                      if(tempCC != null)
                        writeToSelf("<who><name>" + tempCC.getName() +
                            "</name><id>" + tempCC.getIdNum() + "</id></who>");
                    }
                  } else {
                    ClientContainer tempCC;
                    for(String client : lookups.split(",")) {
                      tempCC = server.clientList.get(Integer.parseInt(client));
                      if(tempCC != null)
                        writeToSelf("<who><name>" + tempCC.getName() +
                            "</name><id>" + client + "</id></who>");
                    }
                  }
                }
              }
              //use the server method to kill self so everything closes cleanly
              server.killClient(idNum);
              break;
              case 2: //whereis to come FIX THIS CRAP
              {
                String tempType = "";
                for(char inner_temp : data.toCharArray()) {
                  ixp.parse(inner_temp);
                  if(ixp.hasData()) {
                    switch(ixp.getTagType()) {
                      case 2: //mtype
                        tempType = ixp.getData();
                        break;
                    }
                    ixp.reset();
                  }
                }
                ixp.reset();
                
                if(!tempType.equalsIgnoreCase("")) {
                  if(tempType.equalsIgnoreCase("all")) {
                    for(ModuleContainer mc : server.moduleList.values()) {
                      writeToSelf("<where><name>" + mc.getName() +
                          "</name><mid>" + mc.getIdNum() + "</mid><mtype>" +
                          mc.getType() + "</mtype></where>");
                    }
                  } else {
                    for(String mtype : tempType.split(",")) {
                      for(ModuleContainer mc : server.moduleList.values()) {
                        if(mc.getType().equalsIgnoreCase(mtype.toLowerCase())) {
                          writeToSelf("<where><name>" + mc.getName() +
                              "</name><mid>" + mc.getIdNum() + "</mid><mtype>" +
                              mc.getType() + "</mtype></where>");
                        }
                      }
                    }
                  }
                } else {
                  //blank errors here?
                }
              }
              //use the server method to kill self so everything closes cleanly
              server.killClient(idNum);
              break;
              case 3: //disconnect
              {
                int id = 0;
                int chNum = 0;
                for(char inner_temp : data.toCharArray()) {
                  ixp.parse(inner_temp);
                  if(ixp.hasData()) {
                    switch(ixp.getTagType()) {
                      case 1: //id
                        id = ixp.getDataAsInt();
                        break;
                      case 4: //key
                        chNum = ixp.getDataAsInt();
                        break;
                    }
                    ixp.reset();
                  }
                }
                ixp.reset();
                
                if(id != 0 && chNum != 0) {
                  server.killClientAuth(id, chNum);
                } else {
                  //add error message here
                }
              }
              server.killClient(idNum);
              break;
            }
          }
          oxp.reset();
        }
      }
    } catch (Exception e) {
      System.out.println("Connection to client: " + idNum + " has failed.  Disconnecting...");
      server.killClient(idNum);
    }
  }
  
//REFLEXIVE FUNCTIONS
  /**This function attempts to cleanly bring down a client,
   * set the calling opject to null after this function has
   * been called to ensure that the GC picks it up.
   * 2006/08/10 Kyle Reed
   */
  public void selfDestruct() {
    try {
      disconnectFromModule();
      myKey.channel().close();
      myKey.cancel();
      System.out.println("Client #" + idNum + " has been disconnected.");
    } catch (Exception e) {
      //The connection was already closed if this block is reached
    }
  }
  
//SETS AND GETS
  /** Returns a reference to the module that this
   *  client is associated with; <b>null</b> if this
   *  client isn't connected to a module. 2006/08/10 Kyle Reed
   */
  public ModuleContainer connectedModule() {
    return moduleCtnr;
  }
}
