package Server;

import java.nio.channels.SelectionKey;
/** The startup class that watches the server threads and kills them
 *  when the server is exiting.  It also launches the server and maintains
 *  the daemon thread. 2006/08/10 Kyle Reed
 */
public class WatchDog extends Thread {
  /** Startup point for the server that starts the server and admin parachute threads.
   *  2006/08/10 Kyle Reed
   */
   public static void main(String[] args) {
    try {
      /* Ok, here's how this works
       * The server is created and passed to the AdminConsole
       * The AdminConsole controls the server.
       * In order to cleanly kill the Console and the Server
       * in the event of one of the parts breaking, the WatchDog
       * monitors the state of the server attached to the
       * AdminConsole.  If the 'kill()' method was called on 
       * the server, the WatchDog brings down everything and
       * cleanly exits.  It's not the cleanest solution but
       * it makes it easy to bring down the whole thing if a
       * part breaks.
       */
      AdminConsole console = new AdminConsole();
      WatchDog wd = new WatchDog(console);
      console.start();
      wd.start();
    } catch(Exception e) {
      System.out.println("Exception caught in ServerStart.");
      System.out.println(e.getMessage());
    }
  }
  
  AdminConsole watch;
  /** Constructor for the watch dog. It takes a reference to the AdminConsole
   *  object that contains the actual server to be watched. 2006/08/10 Kyle Reed
   */
  public WatchDog(AdminConsole s) {
    watch = s;
  }  
  
  /** Daemon thread of the WatchDog.  Checks every 2 seconds to see if a part
   *  has set the Server.alive flag to <i>false</i>. If it's <i>false</i> then
   *  it runs the shutdown routine that cleans up the connections threads.
   *  2006/08/10 Kyle Reed
   */
  public void run() {
    try {
      while(watch.getServer().isAlive())
        sleep(2000);
    } catch (InterruptedException ie) {
      System.out.println("WatchDog was interrupted.  Terminating Server...");
    } catch (Exception e) {
      System.out.println("WatchDog encountered an error. Terminating Server...");
    } finally {
      closeModules();
      //we are trying to shutdown nicely because... it's better than just killing the VM
      if(!closeSocketListener() && closeRemoteAdmin()) {
        System.out.println("Could not shutdown cleanly. Killing server...");
        System.exit(0);
      }
    }
    System.out.println("Server has been shutdown.");
  }
  
  /** Tells the server to send close messages to the modules so they
   *  should exit nicely, allowing a restart of the server. 2006/08/10 Kyle Reed
   */
  private void closeModules() {
    watch.getServer().killAllModules();
  }
  /** Attempts to nicely close the main server listener in order to close
   *  all of the persistant connections. Returns <i>true</i> if the listener
   *  closes nicely. 2006/08/10 Kyle Reed
   */
  private boolean closeSocketListener() {
    try {
      if(watch.getServer().getSelector().isOpen())
        watch.getServer().getSelector().close();
      return true;
    } catch (Exception e) {
      System.out.println("Exception closing socketListener");
      return false;
    }
  }
  /** Attempts to nicely close the AdminConsole.
   *  Returns <i>true</i> if the AdminConsole closes nicely.
   *  2006/08/10 Kyle Reed
   */
  private boolean closeRemoteAdmin() {
    try {
      watch.getServerSocket().close();
      return false;
    } catch (Exception e) {
      System.out.println("Exception closing remoteAdminServer");
      return false;
    }
  }
}